/*
 * ex4.c : a simple __char device driver__
 *
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */


/*
 * Includes
 */

#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/devfs_fs_kernel.h>	/* devfs support */

#include "ex.h"


/*
 * Arguments
 */

static int major = 0; /* Major number */

MODULE_DESCRIPTION("Simple __char device driver__ (ex4)");
MODULE_AUTHOR("Julien Gaulmin, Alcove");
MODULE_SUPPORTED_DEVICE("Nothing in particular");
MODULE_PARM_DESC(major, "Static major number (none = dynamic)");
MODULE_PARM(major, "i");


/*
 * Global variables
 */

static devfs_handle_t devfs_handle; /* Devfs file handle */


/*
 * File operations
 */

static ssize_t ex4_read(struct file *file, char *buf, size_t count, loff_t *ppos)
{
	printk(KERN_DEBUG "ex4_read()\n");

	return count;
}


static ssize_t ex4_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
	printk(KERN_DEBUG "ex4_write()\n");

	return count;
}


static int ex4_open(struct inode *inode, struct file *file)
{
	printk(KERN_DEBUG "ex4_open()\n");

	return 0;
}


static int ex4_release(struct inode *inode, struct file *file)
{
	printk(KERN_DEBUG "ex4_release()\n");

	return 0;
}


static struct file_operations ex4_fops = {
	owner:		THIS_MODULE,
	read:		ex4_read,
	write:		ex4_write,
	open:		ex4_open,
	release:	ex4_release,
};


/*
 * Init and Exit
 */

static int __init ex4_init(void)
{
	int ret;

	ret = devfs_register_chrdev(major, "ex4", &ex4_fops);
	if (ret < 0) {
		printk(KERN_WARNING "Unable to get a major for ex4");

		return ret;
	}

	if (major == 0)
		major = ret; /* dynamic value */

	devfs_handle = devfs_register(NULL, "ex4", DEVFS_FL_DEFAULT, major, 0, S_IFCHR | S_IRUGO | S_IWUSR, &ex4_fops, NULL);

	printk(KERN_INFO "ex4 successfully loaded with major %d\n", major);

	return 0;
}


static void __exit ex4_exit(void)
{
	int ret;

	ret = devfs_unregister_chrdev(major, "ex4");
	if (ret < 0)
		printk(KERN_WARNING "Error while unregistering ex4");

	devfs_unregister(devfs_handle);

	printk(KERN_INFO "ex4 successfully unloaded\n");
}


/*
 * Module entry points
 */

module_init(ex4_init);
module_exit(ex4_exit);
