/*
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */

#include <sys/types.h>	/* open() & fork() */
#include <sys/stat.h>	/* open() */
#include <fcntl.h>	/* open() */
#include <unistd.h>	/* fork() & sleep() & read() & close() */
#include <stdio.h>	/* sscanf() & perror() */
#include <stdlib.h>	/* atoi() & malloc() */


#define	PAGE_SIZE	4096
#define	BUFFER_SIZE	10 * PAGE_SIZE


int do_read(char *filename, int fd, int count)
{
	int ret, try=0, got=0;
	char *buffer;

	if (count < 0 || count > BUFFER_SIZE) {
		printf("Size should be less than %dko\n", BUFFER_SIZE / 1024);

		return 1;
	}
	
	buffer = (char *)malloc(count + 1);
	if (buffer == NULL) {
		printf("Unable to allocate %d bytes", count);

		return 1;
	}

	while (got < count) {
		int new_count = count - got;
		
		ret = read(fd, buffer + got, new_count);
		if (ret < 0) {
			printf("Error doing \'read %s %d\'\n",
				filename, new_count);
			perror("read()");

			return 1;
		} else if (ret == 0) {
			printf("\nCommand \'read %s %d\' ==> "
				"%d chars in %d time\n",
				filename, count, got, try);

			break;
		} else {
			buffer[got + ret] = '\0';
			printf("%s", buffer + got);

			got += ret;
			try++;
		}
	}

	if (ret != 0) {
		printf("\nCommand \'read %s %d\' ==> %d chars in %d time\n",
				filename, count, got, try);
	}

	ret = close(fd);
	if (fd < 0) {
		printf("Unable to close file descriptor %d\n", fd);
		perror("close()");

		return 1;
	}

	return 0;
}


int main (int argc, char **argv)
{
	int fd, ret;

	if (argc != 3) {
		printf("Usage: %s <device> <size>\n", argv[0]);

		return 1;
	}

	fd = open((char *)argv[1], O_RDWR);
	if (fd < 0) {
		printf("Unable to open %s\n", argv[1]);
		perror("open()");

		return 1;
	}

	ret = fork();
	switch (ret) {
		case -1:
			printf("Unable to fork\n");
			perror("fork()");

			return 1;

		case 0:
			printf("Son\n");

			do_read(argv[1], fd, atoi(argv[2]));

			printf("Son done\n");

			sleep(10);

			printf("End of son\n");

			break;

		default:
			printf("Father\n");

			do_read(argv[1], fd, atoi(argv[2]));

			printf("Father done\n");

			sleep(20);

			printf("End of father\n");

			break;
	}

	return 0;
}
