/*
 * ex11.c : a simple replacement for keyboard interrupt handler. Based on
 *          intrpt.c from Ori Pomerantz. 
 *
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */


/*
 * Includes
 */

#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit} */
#include <linux/tqueue.h>	/* bottom halves */
#include <linux/interrupt.h>	/* interrupts */
#include <asm/io.h>		/* {in,out}b */


/*
 * Arguments
 */

MODULE_DESCRIPTION("Simple replacement for keyboard interrupt handler (ex11)");
MODULE_AUTHOR("Julien Gaulmin, Alcove");
MODULE_SUPPORTED_DEVICE("Nothing in particular");


/*
 * Constantes
 */

#define KBD_IRQ	1


/*
 * Global variables
 */

static struct tq_struct kbd_bh_task;
static unsigned char scancode;


/*
 * Event handlers
 */

static void kbd_bh_handler(void *code)
{
	printk("ex11: scancode = %x -> %s.\n",
		(int) *((char *)code) & 0x7F,
		*((char *)code) & 0x80 ? "RELEASED" : "PRESSED");
}


static void new_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	/* Read keyboard status and then the scancode */
	inb(0x64);
	scancode = inb(0x60);

	/* Prepare a task */
	PREPARE_TQUEUE(&kbd_bh_task, kbd_bh_handler, &scancode);

	/* Add a task to the immediate bottom half and mark it for schedule */
	queue_task(&kbd_bh_task, &tq_immediate);
	mark_bh(IMMEDIATE_BH);
}


/*
 * Init and Exit
 */

static int __init ex11_init(void)
{
	int ret;

	/* Initialize task */
	INIT_TQUEUE(&kbd_bh_task, NULL, NULL);

	/* Uninstall the previous keyboard handler */
	free_irq(KBD_IRQ, NULL);

	/* Install our own keyboard handler */
	ret = request_irq(KBD_IRQ, new_irq_handler, SA_SHIRQ, "new_keyboard", &scancode);  
	if (ret < 0) {
		printk(KERN_WARNING "ex11: unable to register the kbd irq\n");

		return ret;
	}

	printk(KERN_INFO "ex11 successfully loaded\n");

	return 0;
}


static void __exit ex11_exit(void)
{
	free_irq(KBD_IRQ, NULL);

	printk(KERN_INFO "ex11 successfully unloaded\n");
}


/*
 * Module entry points
 */

module_init(ex11_init);
module_exit(ex11_exit);
