/*
 * ex2.c : a simple module with arguments and __/proc__
 *
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */


/*
 * Includes
 */

#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/proc_fs.h>	/* proc fs */

#include "ex.h"


/*
 * Arguments
 */

static char *id = "ex2_default"; /* ID-String of the module */
static int num = 1234; /* A number */

MODULE_DESCRIPTION("Simple module with arguments and __/proc__ (ex2)");
MODULE_AUTHOR("Julien Gaulmin, Alcove");
MODULE_SUPPORTED_DEVICE("Nothing in particular");
MODULE_PARM_DESC(id, "ID-String of the driver");
MODULE_PARM_DESC(num, "A number");
MODULE_PARM(id, "1-" __MODULE_STRING(MAX_ID_SIZE) "s");
MODULE_PARM(num, "i");


/*
 * /proc handling
 */

static int ex2_get_info(char *page, char **start, off_t off, int count)
{
	int len = 0;

	len += sprintf(page + len, "ex2 arguments :\n");
	len += sprintf(page + len, "id=%s\n", id);
	len += sprintf(page + len, "num=%d\n", num);

	return len;
}


/*
 * Init and Exit
 */

static int __init ex2_init(void)
{
	struct proc_dir_entry *proc_entry;

	printk(KERN_INFO "ex2 initialization\n");
	printk(KERN_INFO "ex2 arguments : id=%s, num=%d\n", id, num);

	proc_entry = create_proc_entry("ex2", 0, NULL);
	if (proc_entry) {
		proc_entry->get_info = ex2_get_info;
	} else {
		printk(KERN_ERR "Can't create proc entry for ex2\n");

		return -EAGAIN;
	}

	return 0;
}


static void __exit ex2_exit(void)
{
	remove_proc_entry("ex2", NULL);

	printk(KERN_INFO "ex2 exit\n");
}


/*
 * Module entry points
 */

module_init(ex2_init);
module_exit(ex2_exit);
