/*
 * ex3.c : a simple module with arguments and __ps like__ in /proc
 *
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */


/*
 * Includes
 */

#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/proc_fs.h>	/* proc fs */

#include "ex.h"


/*
 * Arguments
 */

static char *id = "ex3_default"; /* ID-String of the module */
static int num = 1234; /* A number */

MODULE_DESCRIPTION("Simple module with arguments and __ps like__ in /proc (ex3)");
MODULE_AUTHOR("Julien Gaulmin, Alcove");
MODULE_SUPPORTED_DEVICE("Nothing in particular");
MODULE_PARM_DESC(id, "ID-String of the driver");
MODULE_PARM_DESC(num, "A number");
MODULE_PARM(id, "1-" __MODULE_STRING(MAX_ID_SIZE) "s");
MODULE_PARM(num, "i");


/*
 * Features
 */

static int ps(char *buf) {
	struct task_struct *p;
	int len = 0;

	len += sprintf(buf + len, "Simple PS :\n");

	read_lock(&tasklist_lock);
	for_each_task(p) {
		if (len + MAX_PS_LINE_SIZE > PAGE_SIZE) {
			len += sprintf(buf + len, "[TRUNCATED]\n");

			break;
		}

		len += sprintf(buf + len, "%d\t", (int)p->pid);
		len += sprintf(buf + len, "%s\n", p->comm);
	}
	read_unlock(&tasklist_lock);

	return len;
}


/*
 * /proc handling
 */

static int ex3_get_info(char *page, char **start, off_t off, int count)
{
	return ps(page);
}


/*
 * Init and Exit
 */

static int __init ex3_init(void)
{
	struct proc_dir_entry *proc_entry;

	printk(KERN_INFO "ex3 initialization\n");
	printk(KERN_INFO "ex3 arguments : id=%s, num=%d\n", id, num);

	proc_entry = create_proc_entry("ex3", 0, NULL);
	if (proc_entry) {
		proc_entry->get_info = ex3_get_info;
	} else {
		printk(KERN_ERR "Can't create proc entry for ex3\n");

		return -EAGAIN;
	}

	return 0;
}


static void __exit ex3_exit(void)
{
	remove_proc_entry("ex3", NULL);

	printk(KERN_INFO "ex3 exit\n");
}


/*
 * Module entry points
 */

module_init(ex3_init);
module_exit(ex3_exit);
