/*
 * ex3bis.c : a simple module with arguments and __ps like__ in /proc
 *
 * Copyright (C) 2001 Julien Gaulmin <julien.gaulmin@fr.alcove.com>, Alcve
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA or look at http://www.gnu.org/copyleft/gpl.html
 */


/*
 * Includes
 */

#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/proc_fs.h>	/* proc fs */

#include "ex.h"


/*
 * Arguments
 */

static char *id = "ex3bis_default"; /* ID-String of the module */
static int num = 1234; /* A number */

MODULE_DESCRIPTION("Simple module with arguments and __ps like__ in /proc (ex3bis)");
MODULE_AUTHOR("Julien Gaulmin, Alcove");
MODULE_SUPPORTED_DEVICE("Nothing in particular");
MODULE_PARM_DESC(id, "ID-String of the driver");
MODULE_PARM_DESC(num, "A number");
MODULE_PARM(id, "1-" __MODULE_STRING(MAX_ID_SIZE) "s");
MODULE_PARM(num, "i");


/*
 * Features
 */

static int ps(char *buf) {
	struct task_struct *p;
	int len = 0;

	len += sprintf(buf + len, "Simple PS :\n");

	read_lock(&tasklist_lock);
	for_each_task(p) {
		if (len + MAX_PS_LINE_SIZE > PAGE_SIZE) {
			len += sprintf(buf + len, "[TRUNCATED]\n");

			break;
		}

		len += sprintf(buf + len, "%d\t", (int)p->pid);
		len += sprintf(buf + len, "%s\n", p->comm);
	}
	read_unlock(&tasklist_lock);

	return len;
}


/*
 * /proc handling
 */

static int ex3bis_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len = ps(page);

	if (len <= off + count)
		*eof = 1;

	*start = page + off;
	len -= off;

	if (len > count)
		len = count;

	if (len < 0)
		len = 0;

	return len;
}


/*
 * Init and Exit
 */

static int __init ex3bis_init(void)
{
	struct proc_dir_entry *proc_entry;

	printk(KERN_INFO "ex3bis initialization\n");
	printk(KERN_INFO "ex3bis arguments : id=%s, num=%d\n", id, num);

	proc_entry = create_proc_read_entry("ex3bis", 0, NULL, ex3bis_read_proc, NULL);
	if (!proc_entry) {
		printk(KERN_ERR "Can't create proc entry for ex3bis\n");

		return -EAGAIN;
	}

	return 0;
}


static void __exit ex3bis_exit(void)
{
	remove_proc_entry("ex3bis", NULL);

	printk(KERN_INFO "ex3bis exit\n");
}


/*
 * Module entry points
 */

module_init(ex3bis_init);
module_exit(ex3bis_exit);
